/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Googlebar for Mozilla.
 *
 * The Initial Developer of the Original Code is Andy Edmonds.
 * Portions created by the Initial Developer are Copyright (C) 2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Andy Boughton
 *  Andrew Houghton
 *  John Woods
 *  Bernd Kuemmerlen 	
 *  Robert Mulcahy
 *  Alfred Kayser
 *  Reflex
 *  Gary Turnbull
 *  Raj Bhaskar
 *  Joachim Thewes
 *  Henrik Gemal
 *  Robert Fernandes
 *  Joe Chellman
 *  Franki Cheung
 *  Alban Fonrouge
 *  Martin Hassman
 *  Ufuk Kayserilioglu
 *  Yoni Gilad 
 *  Tim Schmidt
 *  timeless
 *  Francis Turner
 *  
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


// ****************************
// *  BEGIN NEWLY REFACTORED  *
// ****************************


// ****************************************
// *  CONSTANTS - SERVICES, OBJECTS, ETC  *
// ****************************************

const GB_STATE_START = Components.interfaces.nsIWebProgressListener.STATE_START;
const GB_STATE_STOP = Components.interfaces.nsIWebProgressListener.STATE_STOP;

const GB_DECK_ACTIONS = [
  "default", "site", "lucky", "groups", "directory", "images", "news", "USA", "Catalog", "dictionary",
  "glossary", "mac", "bsd", "linux", "microsoft", "maps", "video", "stocks", "froogle", "university", 
  "nocalc", "gb-mycountry", "gb-mylocal", "gblogger", "scholar", "answers", "desktop", "print"
];

const GB_DECK_LOCS = [ "Same Window", "New Tab", "New Window" ];

const GB_THUMBNAIL_URL = "http://pthumbnails.alexa.com/image_server.cgi?id=googlebar.mozdev.org&size=small&url=";
const GB_USA_ONLY_DOMAIN = "com";
const GB_UNIV_URL = "http://www.google.com/options/universities.html";


// **************************
// *  MISC SUPPORT OBJECTS  *
// **************************


var myGooglebarFocusListener = {

	// Stateful Show/Focus: interact with the GoogleBar given its current state
	// using a single keyboard shortcut.  The possible state transitions are
	//
	//  - If the toolbar is hidden, toggle (show) it and focus on the criteria box
	//
	//  - If the criteria box is in focus and ...
	//     - ... it has no text, fill in and select a default tooltip
	//     - ... it has less than all of its text selected, select all its text
	//     - ... it has all of its text selected, toggle (hide) the toolbar
	//
	//  - If the criteria box is blurred, then focus on it
	//
	// Before focus is given to the criteria box, the element with the current
	// focus is saved.  When the toolbar is then later hidden, focus is restored
	// to that saved element.

	mPrevFocus: null,

	getFocusedElement: function() {
		var ff = document.getElementsByAttribute("focused",true);
		return ff.length ? ff[0] : null;
	},
	
	statefulShowFocus: function() {
		var googlebar = myGooglebarUtil.getElement( "googlebar" );
		var critBox = myGooglebarUtil.getElement( "ggb_search_criteria" );
	
		if ( googlebar.hidden ) {
			this.mPrevFocus = this.getFocusedElement();
			goToggleToolbar( "googlebar", "ggb_cmd_ShowHide" );
			critBox.focus();
		}
		else if ( critBox.getAttribute("focused") ) {	
			if ( critBox.inputField.textLength == 0 ) {
				critBox.value = "GoogleBar \"Enter your search text\"";
				critBox.inputField.select();
			}
			else {
				var curClickStart = critBox.inputField.selectionStart;
				var curClickEnd = critBox.inputField.selectionEnd;
				var curSelSize = curClickEnd - curClickStart;

				if ( curSelSize < critBox.inputField.textLength ) {
					critBox.inputField.select();
				}
				else {
					if( this.mPrevFocus != null ) {
						this.mPrevFocus.focus();
					}
					else {
						window.content.focus();
					}
					goToggleToolbar( "googlebar", "ggb_cmd_ShowHide" );
				}
			}
		}
		else {
			this.mPrevFocus = this.getFocusedElement();
			critBox.focus();
		}
	}
};

var myGooglebarLocListener = {

	QueryInterface: function(aIID) {
		if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
		aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
		aIID.equals(Components.interfaces.nsISupports))
		return this;
		throw Components.results.NS_NOINTERFACE;
	},

	onLocationChange: function(aWebProgress, aRequest, aLocation) {
	    if (aLocation) {
	        var url = aLocation.spec;
	        var terms;
	
			// Old (broken) tests
			//if(/^http:\/\/[^\.]*\.google\..*\/[^\?]*\?.*q=([^&]*)&.*/i.test(url))
			//if(/^http:\/\/[^\.]*\.google\..*?\/[^\?]*?\?.*?q=([^&]*)/i.test(url))

			// Test to see if the URL matches the given pattern
			if (/^http:\/\/[^\.]*?\.google\..*?\/[^\?]*?(?:\?q=|\?.*?&q=)([^&]*)&?.*/i.test(url)) {
	            terms = RegExp.$1;
	            terms = terms.replace(/\+?(cache|link|related|stocks|define|site|allintitle|intitle|allinurl|inurl|filetype|group):[^\+]*\+?/g, "");
	            terms = terms.replace(/\+/g, " ");
	            terms = terms.replace(/\#.*$/g, "");
	            terms = decodeURIComponent(terms);
	
	            var critBox = myGooglebarUtil.getElement("ggb_search_criteria");
	            critBox.value = terms;
	
	            myGooglebar.refreshSearchButtons();
	        }
	        else {
	        	myGooglebar.updateNewWindow();
	        	myGooglebar.refreshSearchButtons();
	        }
	
			// Always allow site search, even against google sites

	        var siteBtn = myGooglebarUtil.getElement("ggb_site_separate");
	        siteBtn.setAttribute("disabled", (url && url.indexOf("file://") != 0) ? false : true);
	        
	        // if (/^\s*http:\/\/[^\.]*\.google\..*\/.*/i.test(url)) {
	        //     if (/^\s*http:\/\/[^\.]*\.google\..*\/[^\?]*\?.*q=site:([^&]*)(&.*)*/i.test(url)) {
	        //         siteBtn.setAttribute("disabled", false);
	        //     }
	        //     else {
	        //         siteBtn.setAttribute("disabled", true);
	        //     }
	        // }
	        // else {
	        //     siteBtn.setAttribute("disabled", false);
	        // }

			// Auto-dectect Desktop Search IDs if possible

			var deskRegex = new RegExp("^" + myGooglebarPrefUtil.getPrefValue(GB_PREF_DESKTOP_URL_NAME) + ".*");
			if (deskRegex.test(url)) {
				var deskID = /.*&s=([^&\?]*).*/.test(url) ? RegExp.$1 : "";
				
				if (url.indexOf("/search") > 0) {
					if (!myGooglebarPrefUtil.getPrefValue(GB_PREF_DESKTOP_SEARCH_ID_NAME)) {
						myGooglebarPrefUtil.setPrefValue(GB_PREF_DESKTOP_SEARCH_ID_NAME, deskID);
					}
				}
				else {
					if (!myGooglebarPrefUtil.getPrefValue(GB_PREF_DESKTOP_ID_NAME)) {
						myGooglebarPrefUtil.setPrefValue(GB_PREF_DESKTOP_ID_NAME, deskID);
					}
				}
			}
	        
	        myGooglebar.updateUp();
	    }
	    
	    return 0;
	},
	
	onStateChange: function (aWebProgress, aRequest, aStateFlags, aStatus) {
		if (aStateFlags & GB_STATE_START) {

		}

		if (aStateFlags & GB_STATE_STOP) {
            if (myGooglebarPrefUtil.getPrefValue(GB_PREF_AUTO_HIGHLIGHT_NAME) && myGooglebarHighlight.isHighlightOn()) {
                myGooglebarHighlight.highLightInPage("on");
            }

			myGooglebar.populateUniv();
            myGooglebar.addThumbnails();
            myGooglebar.updateMaps();
            myGooglebar.updateNavigation();
		}
		
		return 0;
	},
	
	onLinkIconAvailable: function() { return 0; },
	onProgressChange: function (aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) { return 0; },
	onSecurityChange: function(aWebProgress, aRequest, aState) { return 0; },
	onStatusChange: function(aWebProgress, aRequest, aStatus, aMessage) { return 0; }
};

var myGooglebarPrefObserver = {

	observe: function(aSubject, aTopic, aData) {
		// aSubject is the nsIPrefBranch being observed
	
		if (aTopic != "nsPref:changed") {
			return;
		}
		
		// These are for debugging only, removed ater testing
		// myGooglebarUtil.logMessage("Pref updated - " + aData);
		// myGooglebarUtil.logMessage("New value - " + myGooglebarPrefUtil.getPrefValue(aData));
		
		switch (aData) {
		
			case GB_PREF_BUTTON_LABELS_NAME:
				myGooglebarUtil.getElement("ggb_ctxt_tbar_LabelsButtonItem").setAttribute("checked", myGooglebarPrefUtil.getPrefValue(GB_PREF_BUTTON_LABELS_NAME));
				myGooglebar.labels();
				break;
			
			case GB_PREF_BUTTON_SEPARATORS_NAME:
				myGooglebarDnD.updateSeparators();
				break;
				
			case GB_PREF_CONTEXT_MENU_NAME:
				myGooglebar.updateContextMenu();
				break;
			
			case GB_PREF_DESKTOP_ID_NAME:
			case GB_PREF_DESKTOP_SEARCH_ID_NAME:
			case GB_PREF_DESKTOP_URL_NAME:
				myGooglebar.updateDesktopItems();
				break;
						
			case GB_PREF_GMAIL_UPDATE_NAME:
				if (myGooglebarPrefUtil.getPrefValue(GB_PREF_GMAIL_UPDATE_NAME)) {
					myGooglebarMail.startPolling();
				}
				else if (!myGooglebarPrefUtil.getPrefValue(GB_PREF_GMAIL_UPDATE_NAME)) {
					myGooglebarMail.stopPolling();
				}
				break;
				
			case GB_PREF_GMAIL_DELAY_NAME:
				myGooglebarMail.setTimeoutDelay(myGooglebarPrefUtil.getPrefValue(GB_PREF_GMAIL_DELAY_NAME));
				break;
			
			case GB_PREF_HIDE_MENU_NAME:
				myGooglebar.syncAllMainPopups();
				myGooglebarDnD.updateSeparators();
			    myGooglebar.refreshSearchButtons();
				break;
			
			case GB_PREF_HISTORY_MAX_NAME:
				myGooglebarHistory.trimHistory();
				break;
				
			case GB_PREF_HISTORY_SAVE_NAME:
				myGooglebar.updateCriteriaDropmarker();
				
				// clear the history when it was switched off
				if (!myGooglebarPrefUtil.getPrefValue(GB_PREF_HISTORY_SAVE_NAME)) {
					myGooglebarHistory.clearHistory( false ); // and clear the critBox
				}
				break;

			case GB_PREF_LOCK_ORDINALS_NAME:
				myGooglebarUtil.getElement("ggb_ctxt_tbar_OrderLockButtonItem").setAttribute("checked", myGooglebarPrefUtil.getPrefValue(GB_PREF_LOCK_ORDINALS_NAME));
				myGooglebar.ordinals();
				break;
				
			case GB_PREF_SAVE_LAST_SEARCH_NAME:
				myGooglebar.resetParents();
				break;

			case GB_PREF_FORM_HIGHLIGHT_NAME:
			case GB_PREF_TERM_COLORS_NAME:
				myGooglebarHighlight.refreshHighlight();
				break;
				
			case GB_PREF_THUMBNAILS_NAME:
				getBrowser().reload();
				break;
		
			default:
				// myGooglebarUtil.logMessage("pref changed -- not paying attention to " + aData + " pref");
				break;
		}
	},
	
	register: function() {
		if (!GB_PREF_OBJ) {
			return;
		}
	
		var pbi = GB_PREF_OBJ.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
		pbi.addObserver("", this, false);
	},
	
	unregister: function() {
		if (!GB_PREF_OBJ) {
			return;
		}
		
		var pbi = GB_PREF_OBJ.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
		pbi.removeObserver("", this);
	}
};

myGooglebarPrefObserver.register();


// ***************************
// *  MAIN GOOGLEBAR OBJECT  *
// ***************************


var myGooglebar = {
	
	mCntryFroogleSfxs: new Array(),
	mCntryMapsSfxs: new Array(),
	mCntryNewsSfxs: new Array(),
	mCntryNewsSites: new Array(),
	mCntrySfxs: new Array(),
	mMyCntrySfxs: new Array(),
	mMyLangSfxs: new Array(),
	
	mHistoryLoaded: false,
	mOpenerLoaded: false,
	mToolbarLoaded: false,
	
	mCritIgnoreClick: false,
	mCritIgnoreFocus: false,
	mCritSelected: false,	

	addTerms: function(aCriteria) {
	    var critBox = myGooglebarUtil.getElement("ggb_search_criteria");
	    var mySearchTermBox = myGooglebarUtil.getElement("ggb_searchterms_box");
	    var mySearchTermMenu = myGooglebarUtil.getElement("ggb_searchterms_menu");
	    var mySearchTermPopup = myGooglebarUtil.getElement("ggb_searchterms_popup");
	    var myTerms = myGooglebarUtil.findTerms(aCriteria);
	    var tempNode, tempValue;

	    var searchBoxWidth = mySearchTermBox.boxObject.width;
	    var searchTermWidth = 0;
	    var availableWidth = this.availableWidth();
	    var overflow = false;
	    
	    mySearchTermBox.setAttribute("googlebaroverflow", false);
	
	    for (var i = 0; i < myTerms.length; i++) {
	        tempNode = document.createElement("text");
	        tempNode.setAttribute("value", myTerms[i]);
	        tempNode.className = "ggb_css_search_term";
	        tempValue = myTerms[i].replace(/\\/, "\\\\");
	        tempNode.setAttribute("onclick", "myGooglebarUtil.findInPage(event, \"" + tempValue + "\")");
	
	        if (myGooglebarHighlight.isHighlightOn()) {
	            // for some reason, the following line does not work (the style attribute is set 
	            // but not honored by mozilla), so we have to use a global class
	            
	            // RF - 2005/01/25
	            // the style seems to be working in newer versions of firefox and mozilla
	            // and we want to use the new modifiable array of colors and not the predetermined
	            // styles in the stylesheet if at all possible.  this likely needs testing.
	            // would be nice to not have to enumerate all 120+ new colors as styles in the
	            // stylesheet if at all possible.
	            
	            tempNode.setAttribute("style", "background-color:" + myGooglebarHighlight.getTermColors()[i%10]);
	            tempNode.className = "ggb_css_search_term";
	        }
	        
	        mySearchTermBox.appendChild(tempNode);
	        searchTermWidth = mySearchTermBox.boxObject.width;
	
	        // check if buttons fit, if not, remove the last button
	        // and add it to the overflow menu
	        if (searchTermWidth > availableWidth) {
	            mySearchTermPopup.appendChild(tempNode);
	            mySearchTermMenu.setAttribute("hidden", false);
	            overflow = true;
	            mySearchTermBox.setAttribute("googlebaroverflow", true);
	        }
	        else {
	            if (mySearchTermBox.getAttribute("googlebaroverflow") == "false") {
	                mySearchTermMenu.setAttribute("hidden", true);
	            }
	        }
	    }
	
	    if (myTerms.length > 0) {
	      myGooglebarUtil.getElement("ggb_highlight_button").setAttribute("disabled", false);
	    }
	    else {
		    mySearchTermMenu.setAttribute("hidden", true);
	    }
	},

	addThumbnails: function() {
		if (!window._content.document || !myGooglebarPrefUtil.getPrefValue(GB_PREF_THUMBNAILS_NAME)) {
			return;
		}
		
		var url = window._content.document.location;
	    if (!(/^http:\/\/[^\.]*?\.google\..*?\/[^\?]*?(?:\?q=|\?.*?&q=)([^&]*)&?.*/i.test(url))) {
	    	return;
	    }
	    
		var contentDoc = window._content.document;
	
		/* Add Thumbnails to Google-Results V0.1 by AlexP */
		var pNode, table, tr, td, attr, href, link, image, nobr;
 
		/* Get Main-Result-Node */
		var pNodes = contentDoc.getElementsByTagName("P");

		/* Loop Results */
		for (var i = 0; i < pNodes.length; i++) {
			pNode = pNodes[i];
			if (pNode.getAttribute("class") != "g") {
				continue;
			}
			if (pNode.parentNode.nodeName == "TD") {
				// myGooglebarUtil.logMessage("Search result already has thumbnails!");
				return;
			}

			link = pNode.getElementsByTagName("A")[0].getAttribute('href');
			// Old regex, broken:   ^http:\/\/[^\.]*?\.google\..*?\/url\/\?sa=U.*\?q=([^&]*).*
			// New regex, by Bernd: ^http:\/\/[^\.]*?\.google\..*?\/url\?sa=U.*&q=([^&]*).*
		    if (/^\/url\?sa=U.*&q=([^&]*).*/i.test(link)) {
		    	link = RegExp.$1;
		    }

			/* Add Table */
			table = contentDoc.createElement("TABLE");
			tr = contentDoc.createElement("TR");
			table.appendChild(tr);

			/* 1st Column */
			td = contentDoc.createElement("TD");
			tr.appendChild(td);
			
			/* Anchor */
			href = contentDoc.createElement("A");
			attr = contentDoc.createAttribute("href");
			attr.nodeValue = link;
			href.setAttributeNode(attr);
						
			/* Image */
			image = contentDoc.createElement("IMG");
			attr = contentDoc.createAttribute("src");
			attr.nodeValue = GB_THUMBNAIL_URL + link;
			image.setAttributeNode(attr);
			
			href.appendChild(image);
			td.appendChild(href);

			/* 2nd Column */
			/* Copy Result to Table-TD */
			td = contentDoc.createElement("TD");
			tr.appendChild(td);
			
			attr = contentDoc.createAttribute("valign");
			attr.nodeValue='top';
			td.setAttributeNode(attr); /* Attributes */

			/* Site Info */
			nobr = pNode.getElementsByTagName("NOBR")[0];
			nobr.appendChild(contentDoc.createTextNode(" --- "));

			href = contentDoc.createElement("A");
			attr = contentDoc.createAttribute("href");
			attr.nodeValue = 'http://partner.alexa.com/amzn/redirect_to_detail?amzn_id=D1EWB5YHDRSMAB&url=' + pNode.getElementsByTagName("A")[0].getAttribute('href');
			href.setAttributeNode(attr);
			attr = contentDoc.createAttribute("class");
			attr.nodeValue = 'fl';
			href.setAttributeNode(attr);		
			href.appendChild(contentDoc.createTextNode(myGooglebarUtil.getProperty("gb.site_info_label")));
			nobr.appendChild(href);

			td.appendChild(pNode.cloneNode(true));

			/* Replace Table with Result */
			pNode.parentNode.replaceChild(table, pNode);
		}
	},

	autoSearch: function(aEvent) {
	    if (myGooglebarPrefUtil.getPrefValue(GB_PREF_AUTO_SEARCH_NAME)) {
	        this.doSearch(aEvent);
	    }
	},

	availableWidth: function() {
	    // This function returns the width which is available for the
	    // search term buttons
	    var width = 0;
	
	    width += myGooglebarUtil.getElement("ggb_parent_container").boxObject.width;
	    width -= myGooglebarUtil.getElement("ggb_search_textbox").boxObject.width;
	    width -= myGooglebarUtil.getElement("ggb_resize_splitter").boxObject.width;
	    width -= myGooglebarUtil.getElement("ggb_toolbar_buttons").boxObject.width;
	    
	    return width;
	},

	clearCriteria: function() {
	    var critBox = myGooglebarUtil.getElement("ggb_search_criteria");
	    critBox.value = "";
	},
	
	clearTerms: function() {
	    var termBox = myGooglebarUtil.getElement("ggb_searchterms_box");
	    var termPopup = myGooglebarUtil.getElement("ggb_searchterms_popup");
	    for (var numTerms = termBox.childNodes.length; numTerms > 0; numTerms--) {
	        termBox.removeChild(termBox.childNodes[0]);
	    }
	    for (numTerms = termPopup.childNodes.length; numTerms > 0; numTerms--) {
	        termPopup.removeChild(termPopup.childNodes[0]);
	    }
	    termBox.setAttribute("googlebaroverflow",false);
	},

	criteriaFocus: function() {
	    var critBox = myGooglebarUtil.getElement("ggb_search_criteria");
	    critBox.focus();
	},

	criteriaKeyEvent: function(aEvent) {
		this.clearTerms();
		this.addTerms(aEvent.target.value);
		
		if (myGooglebarPrefUtil.getPrefValue(GB_PREF_AUTO_HIGHLIGHT_NAME)) {
			myGooglebarHighlight.delayHighlight();
		}
		
		myGooglebarSuggest.suggest(aEvent);
	},

	criteriaSelect: function(aEvent) {
		if (!myGooglebarPrefUtil.getPrefValue(GB_PREF_AUTO_SELECT_NAME)) { return; }
		
	    var critBox = myGooglebarUtil.getElement("ggb_search_criteria");
	    var ePt = critBox.inputField.textLength;
	    var curClickStart = critBox.inputField.selectionStart;
	    var curClickEnd = critBox.inputField.selectionEnd;
	    var curSelSize = curClickEnd - curClickStart;
	
	    if (aEvent && ePt > 0) {
	        switch(aEvent.type) {
	
	        case "dblclick":
	            if (!this.mCritSelected) {
	                critBox.inputField.select();
	                this.mCritSelected = true;
	            }
	            else {
	                critBox.inputField.setSelectionRange(curClickStart, curClickEnd);
	                this.mCritSelected = false;
	            }
	            break;
	
	        case "focus":
	            if (this.mCritIgnoreFocus) {
	                this.mCritIgnoreFocus = false;
	            }
	            else {
	                if (this.mCritSelected) {
	                    if (myGooglebarPrefUtil.getPrefValue(GB_PREF_KEY_SELECTION_NAME)) {
	                        critBox.inputField.setSelectionRange(0, 0 );
	                        this.mCritSelected = false;
	                    }
	                    else {
	                        critBox.inputField.select();
	                    }
	                }
	                else {
	                    critBox.inputField.select();
	                    this.mCritSelected = true;
	                }
	            }
	            break;
	
	        case "mousedown":
	            if (critBox.getAttribute("focused")) {
	                this.mCritIgnoreClick = true;
	            }
	            else {
	                this.mCritIgnoreFocus = true;
	                this.mCritIgnoreClick = false;
	                critBox.inputField.setSelectionRange(0, 0 );
	                this.mCritSelected = true;
	            }
	            break;
	
	        case "click":
	            if (!this.mCritIgnoreClick && curSelSize == 0) {
	                critBox.inputField.select();
	                this.mCritSelected = true;
	            }
	            break;
	
	        default:
	            break;
	
	        }
	    } // no event or text
	},

	criteriaTooltip: function() {
	    var critBox = myGooglebarUtil.getElement("ggb_search_criteria");
	
	    if (critBox.value) {
	        critBox.tooltipText = critBox.value;
	    }
	    else {
	        critBox.tooltipText = critBox.statusText;
	    }
	},

	displayCheck: function() {
	    // make sure that we are loading the preferences
	    this.initialize();
	
	    if (!this.mHistoryLoaded) {
	        this.mHistoryLoaded = true;
	
	        if (myGooglebarPrefUtil.getPrefValue(GB_PREF_HISTORY_SAVE_NAME)) {
	            myGooglebarHistory.initialize();
	        }
	        else {
	            myGooglebarPrefUtil.setPrefValue(GB_PREF_HISTORY_TERMS_NAME, GB_PREF_HISTORY_TERMS_VALUE);
	        }
	    }
	
		myGooglebarHighlight.displayCheck();
	
	    myGooglebarNavigation.loadResults();
	    this.updateCriteriaDropmarker();
	},

	doAction: function(aSearchURL, aLocaton) {
	    switch(aLocaton) {
	
		    case "Same Window":
		        window._content.document.location = aSearchURL;
		        window.content.focus();
		        break;
		
		    case "New Tab":
		        var browser = getBrowser();
                        // Taomyn - replacement code for getReferrer function
                        var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
                        var loc;
                        loc = document.location;
                        var ref = ioService.newURI(loc, null, null);
                        var tab = browser.addTab(aSearchURL, ref);
		        window.content.focus();
		
		        if (!GB_ROOT_OBJ.getBoolPref("browser.tabs.loadBookmarksInBackground")) {
		            browser.selectedTab = tab;
		        }
		        break;
		
		    case "New Window":
		        window.open(aSearchURL);
		        break;
		
		    default:
		        alert(myGooglebarUtil.getProperty("gb.error"));
		        break;
	    }
	},

	doNonSearch: function(aEvent, aSearchType, aSearchParams) {
		// a place to do non Google searches explicitly
		// return true if we find anything we can handle, whether we actually do anything
		// with it is not relevant, just whether we should keep going through the main search
		// processing
		
		/*
		if (aSearchType == 'gb-prefill') {
			if (myGooglebarUtil.isInFirefox()) {
				alert(myGooglebarUtil.getProperty("gb.gbDoNonSearch.1"));		
			}
	
			try {
				const GB_WALLET_SVC = Components.classes["@mozilla.org/wallet/wallet-service;1"].getService(Components.interfaces.nsIWalletService);
			}
			catch (e) {
				myGooglebarUtil.logMessage("Failed to initialize wallet service\n" + e);
			}
	
			var gbformsArray = content.document.forms;

			if (!gbformsArray || !GB_WALLET_SVC) {
				return true;
			}
			
		    if (aEvent.button == 1) {
				GB_WALLET_SVC.WALLET_RequestToCapture(window._content);
			}
			else {
				if (aEvent.ctrlKey || aEvent.shiftKey || aEvent.altKey) {
					// any modifier key bring up the forms dialog if an unknown form,
					// known forms it will fill in if you told it to do so previously
					// do we want to touch passwords and the security issues.
					formPrefill();
				}
				
				try {
					walletService.WALLET_Prefill(true, window._content);
				}
				catch (e) {
				} 
			}

			return true;
		}
		*/

		return false; // we found nothing to handle here, keep going with main googlebarSearch
	},

	doSearch: function(aEvent, aSearchType, aSearchParams) {
	    var win = window._content.document;
	    var searchURL;
	
	    if (!aSearchType) {
	        aSearchType = this.getActFromEvent(aEvent);
	    }
	
		// Some things cannot be done in a new tab, such as formfilling and capturing.
	    // bailing out beforehand
		if (aSearchParams == "NO_SEARCH") {
			if (this.doNonSearch(aEvent, aSearchType, aSearchParams)) {
				return; // we are already done
			}
		}

	    this.clearTerms();
	    
	    var critVal;
	    if (myGooglebarPrefUtil.getPrefValue(GB_PREF_PAGE_SELECTION_NAME)) {
	    	critVal = myGooglebarUtil.getSelectedText(true);
	    	critVal = myGooglebarUtil.trimString(critVal).replace(/\s+/g, " ");
	    	
	    	if (critVal) {
		    	myGooglebarUtil.clearSelectedText();
		    }
	    }

		if (!(critVal && critVal.length > 0)) {
	    	critVal = myGooglebarUtil.trimString(myGooglebarUtil.getElement("ggb_search_criteria").value);
	    	
	    	if (myGooglebarPrefUtil.getPrefValue(GB_PREF_WEB_NAVIGATION_NAME) && this.lookForURL()) {
				// TODO: need to escape illegal characters
				aSearchType = "doUrlNav";
			}
	    }
	    
	    var noCriteria = true;
	    
	    if (critVal && critVal.length > 0) {
	    	noCriteria = false;
	    	this.addTerms(critVal);
	    	
	    	if (aSearchType != "dictionary" || myGooglebarPrefUtil.getPrefValue(GB_PREF_HISTORY_DICT_NAME)) {
	        	myGooglebarHistory.addToHistory(critVal);
	        }
	    }

		var crit = myGooglebarUtil.buildURI(critVal);
		var globalCrit = noCriteria ? "" : myGooglebarPrefUtil.getPrefValue(GB_PREF_GLOBAL_CRITERIA_NAME);
		
	    if (globalCrit != "") {
	    	// TODO: Escape '-' signs?
	    	globalCrit = "+" + myGooglebarUtil.buildURI(globalCrit);
	    }

	    var cntry2Srch = this.mCntrySfxs[myGooglebarPrefUtil.getPrefValue(GB_PREF_COUNTRY_SEARH_NAME)];
	    var cntryMaps2Srch = this.mCntryMapsSfxs[myGooglebarPrefUtil.getPrefValue(GB_PREF_COUNTRY_MAPS_SEARH_NAME)];
	    var cntryFroogle2Srch = this.mCntryFroogleSfxs[myGooglebarPrefUtil.getPrefValue(GB_PREF_COUNTRY_FROOGLE_SEARH_NAME)];
	
	    switch(aSearchType) {
		
		    case "site":
		        var searchHost;
		
		        // If you are on a site search result then reuse that site for the next search,
		        // otherwise use the current window location shown in the browser
		
		        if (/^\s*http:\/\/[^\.]*\.google\..*\/[^\?]*\?.*q=site:([^\+]*)([^&]*)(&.*)*/i.test(win.location.href)) {
		            searchHost = RegExp.$1;
		        }
		        else {
		            searchHost = win.location.hostname;
		        }
		
		        if (noCriteria) {
		        	searchURL = 'http://www.google.' + cntry2Srch + '/search?sourceid=mozclient&ie=utf-8&oe=utf-8&q=site:' + searchHost;
		        }
		        else {
			        searchURL = 'http://www.google.' + cntry2Srch + '/search?sourceid=mozclient&ie=utf-8&oe=utf-8&q=site:' + searchHost + '+' + crit + globalCrit;
			    }
		        break;
		
		    case "backwards":
		        searchURL = 'http://www.google.' + cntry2Srch + '/search?sourceid=mozclient&q=link:' + this.getInfoLoc();
		        break;
		
		    case "similar":
		        searchURL = 'http://www.google.' + cntry2Srch + '/search?sourceid=mozclient&q=related:' + this.getInfoLoc();
		        break;
		
		    case "translate":
		        searchURL = 'http://translate.google.' + GB_USA_ONLY_DOMAIN + '/translate?sourceid=mozclient&u=' + this.getInfoLoc();
		        break;
		
		    case "cachelink":
		        if (!aSearchParams) {
		          aSearchParams = 'http://www.google.com/';
		        }
		        
		        searchURL = 'http://www.google.' + cntry2Srch + '/search?q=cache:' + aSearchParams;
		        break;
		
		    case "cache":
		        searchURL = 'http://www.google.' + cntry2Srch + '/search?q=cache:' + this.getInfoLoc();
		        break;
		
		    case "images":
		       	if (noCriteria)
					searchURL = 'http://images.google.' + cntry2Srch + '/';
		    	else
		        	searchURL = 'http://images.google.' + cntry2Srch + '/images?sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit;
		        	
		        break;
		        
		    case "video":
		    	if (noCriteria)
		    		searchURL = 'http://video.google.' + GB_USA_ONLY_DOMAIN + '/';
		    	else
		    		searchURL = 'http://video.google.' + GB_USA_ONLY_DOMAIN + '/videosearch?sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit;
		    		
		    	break;
		
		    case "groups":
		    	if (noCriteria)
		    		searchURL = 'http://groups.google.' + cntry2Srch + '/';
		    	else
		            searchURL = 'http://groups.google.' + cntry2Srch + '/groups?sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit;
		            
		        break;
		
		    case "directory":
		    	if (noCriteria)
		    		return;
		    	else
		    		crit += globalCrit;
		    		
		        searchURL = 'http://www.google.' + cntry2Srch + '/search?sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit + '&cat=gwd/Top&sa=N&tab=gd';
		        break;
		
		    case "lucky":
		    	if (noCriteria)
		    		return;
		    	else
		    		crit += globalCrit;
		    
		        searchURL = 'http://www.google.' + cntry2Srch + '/search?sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit + '&btnI=I%27m+Feeling+Lucky';
		        break;
		
		    case "scholar":
		    	if (noCriteria)
		    		searchURL = 'http://scholar.google.' + GB_USA_ONLY_DOMAIN + '/';
		    	else
		            searchURL = 'http://scholar.google.' + GB_USA_ONLY_DOMAIN + '/scholar?sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit;
		            
		        break;

		    case "print":
		    	if (noCriteria)
		    		searchURL = 'http://print.google.' + GB_USA_ONLY_DOMAIN + '/';
		    	else
		            searchURL = 'http://print.google.' + GB_USA_ONLY_DOMAIN + '/print?sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit;
		            
		        break;

		    case "desktop":
		    	var deskURL = myGooglebarPrefUtil.getPrefValue(GB_PREF_DESKTOP_URL_NAME);
		    	if (deskURL.substring(deskURL.length - 1) != '/') {
		    		deskURL += '/';
		    	}
		    	
		    	var deskID = myGooglebarPrefUtil.getPrefValue(GB_PREF_DESKTOP_ID_NAME);
		    	var deskSearchID = myGooglebarPrefUtil.getPrefValue(GB_PREF_DESKTOP_SEARCH_ID_NAME);
		    
		    	if (noCriteria)
		    		searchURL = deskURL + '&s=' + deskID;
		    	else
		            searchURL = deskURL + 'search&s=' + deskSearchID + '?q=' + crit;
		            
		        break;
		
		    case "USA":
		    	if (noCriteria)
		    		searchURL = 'http://www.google.' + GB_USA_ONLY_DOMAIN + '/unclesam';
		    	else
		        	searchURL = 'http://www.google.' + GB_USA_ONLY_DOMAIN + '/unclesam?sourceid=mozclient&restrict=unclesam&ie=utf-8&oe=utf-8&q=' + crit;
		        	
		        break;
		
		    case "Catalog":
		    	if (noCriteria)
		    		searchURL = 'http://catalogs.google.' + GB_USA_ONLY_DOMAIN + '/';
		    	else
		            searchURL = 'http://catalogs.google.' + GB_USA_ONLY_DOMAIN + '/catalogs?sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit;
		            
		        break;
		
		    case "bsd":
		    	if (noCriteria)
		    		searchURL = 'http://www.google.' + cntry2Srch + '/bsd';
		    	else
		            searchURL = 'http://www.google.' + cntry2Srch + '/bsd?restrict=bsd&ie=utf-8&oe=utf-8&q=' + crit;
		            
		        break;
		
		    case "microsoft":
		    	if (noCriteria)
		        	searchURL = 'http://www.google.' + cntry2Srch + '/microsoft';
		    	else
		        	searchURL = 'http://www.google.' + cntry2Srch + '/microsoft?sourceid=mozclient&hq=microsoft&restrict=microsoft&ie=utf-8&oe=utf-8&q=' + crit;
		        	
		        break;
		
		    case "mac":
		       	if (noCriteria)
		        	searchURL = 'http://www.google.' + cntry2Srch + '/mac';
		    	else
		        	searchURL = 'http://www.google.' + cntry2Srch + '/mac?sourceid=mozclient&restrict=mac&ie=utf-8&oe=utf-8&q=' + crit;
		
		        break;
		
		    case "linux":
		    	if (noCriteria)
		        	searchURL = 'http://www.google.' + cntry2Srch + '/linux';
		    	else
		            searchURL = 'http://www.google.' + cntry2Srch + '/linux?sourceid=mozclient&restrict=linux&ie=utf-8&oe=utf-8&q=' + crit;
		            
		        break;
		
		    case "news":
				if (noCriteria)        
		            searchURL = this.getNewsLoc();
		        else
		        	searchURL = this.getNewsSearchLoc() + '&sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit;
		        	
		        break;
		
		    case "answers":
		    	if (noCriteria)
		    		searchURL = 'http://answers.google.' + GB_USA_ONLY_DOMAIN + '/';
		    	else
		            searchURL = 'http://answers.google.' + GB_USA_ONLY_DOMAIN + '/answers/main?cmd=search&qtype=all&q=' + crit;
		            
		        break;
		
		    case "stocks":
		    	// If there is no serach criteria then navigate to the ticker entry page
		    	//
		    	// if (noCriteria)
		    	//	return;
		
		        searchURL = 'http://www.google.' + cntry2Srch + '/search?sourceid=mozclient&ie=utf-8&oe=utf-8&q=stocks:' + crit;
		        break;
		
		    case "froogle":
		    	if (noCriteria)
		        	searchURL = 'http://froogle.google.' + cntryFroogle2Srch + '/';
				else
					searchURL = 'http://froogle.google.' + cntryFroogle2Srch + '/froogle?q=' + crit + '&btnG=Froogle+Search&ie=utf-8&oe=utf-8';
		
		        break;
		
		    case "dictionary":
		    	if (noCriteria)
		            searchURL = 'http://www.dictionary.com/';
		        else
		        	searchURL = 'http://www.dictionary.com/cgi-bin/dict.pl?term=' + crit;
		        
		        break;
		
		    case "glossary":
		       	if (noCriteria)
		    		return;
	
		        // Glossary search doesn't appear to be valid for countries outside the US
				// searchURL = 'http://labs.google.' + GB_USA_ONLY_DOMAIN + '/glossary?q=' + crit;
		        searchURL = 'http://www.google.' + GB_USA_ONLY_DOMAIN + '/search?sourceid=mozclient&ie=utf-8&oe=utf-8&q=define:' + crit;
		        break;

		    case "maps":
		    	if (noCriteria)
		    		searchURL = 'http://maps.google.' + cntryMaps2Srch + '/';
		    	else
		    		searchURL = 'http://maps.google.' + cntryMaps2Srch + '/maps?oi=map&sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit;

		    	break;
		
		    case "gb-mycountry":
		       	if (noCriteria)
		    		return;
		    
		    	var mycntry2Srch = this.mMyCntrySfxs[myGooglebarPrefUtil.getPrefValue(GB_PREF_MY_COUNTRY_SEARH_NAME)];
		    	var mylang2Srch = this.mMyLangSfxs[myGooglebarPrefUtil.getPrefValue(GB_PREF_MY_LANGUAGE_SEARH_NAME)];
		    	
		        searchURL = 'http://www.google.' + cntry2Srch + '/search?sourceid=mozclient&cr=' + mycntry2Srch + '&lr=' + mylang2Srch + '&q=' + crit ;
		        break;
		
		    case "gb-mylocal":
		    	var myLocal = myGooglebarPrefUtil.getPrefValue(GB_PREF_MY_LOCAL_NAME);
		    	
		    	if (noCriteria && !myLocal) {
		    		searchURL = 'http://local.google.' + cntry2Srch + '/';
		    	}
		    	else if (noCriteria) {
		    		searchURL = 'http://local.google.' + cntry2Srch + '/local?&near=' + myLocal + '&sourceid=mozclient&ie=utf-8&oe=utf-8';
		    	}
				else {
		        	searchURL = 'http://local.google.' + cntry2Srch + '/local?&near=' + myLocal + '&sourceid=mozclient&ie=utf-8&oe=utf-8&q=' + crit;
				}
	        	break;
		
			case "gblogger":	
				var Q = '';
				var x = window._content; // was document, moz complains
				var y = window._content;
			    
				if( myGooglebarPrefUtil.getPrefValue(GB_PREF_BLOG_STD_WIN_NAME) ){
					if (x.selection)
						Q = x.selection.createRange().text;
					else if (y.getSelection)
						Q = y.getSelection();
					else if (x.getSelection)
						Q = x.getSelection();
					
					// void(window.open('http://www.blogger.com/blog_this.pyra?t='+escape(Q)+'&u='+escape(location.href)+'&n='+escape(document.title)));
					void(window.open('http://www.blogger.' + GB_USA_ONLY_DOMAIN + '/blog_this.pyra?t='+escape(Q)+'&u='+escape(x.location.href)+'&n='+escape(x.document.title),'bloggerForm','scrollbars=no,width=475,height=300,top=175,left=75,status=yes,resizable=yes'));
		
					return;
				}
				else {
					if (x.selection) //this is deprecated, I haven't seen it true yet either
						Q = x.selection.createRange().text;	
					else if (y.getSelection)
						Q = y.getSelection();
					else if (x.getSelection)
						Q = x.getSelection();
		
					searchURL = 'http://www.blogger.' + GB_USA_ONLY_DOMAIN + '/blog_this.pyra?t='+escape(Q)+'&u='+escape(x.location.href)+'&n='+escape(x.document.title);
				}		    
				break;
		
		    case "university":
		    	var university = myGooglebarPrefUtil.getPrefValue(GB_PREF_UNIVERSITY_ID_NAME);
		    
		        if (!university) {
		            alert(myGooglebarUtil.getProperty("gb.button.univ.none"));
		            return;
		        }
		        
		        if (noCriteria) {
		        	searchURL = university;
		        }
		        else if (myGooglebarPrefUtil.getPrefValue(GB_PREF_UNIVERSITY_LABEL_NAME) == "All Universities") {
		            searchURL = university + '&q=' + crit;
		        }
		        else {
		            searchURL = university + '?q=' + crit;
		        }
		        
		        break;
		
		    case "nocalc":
		    	if (noCriteria)
		    		return;
		    
		        searchURL = 'http://www.google.' + cntry2Srch + '/search?sourceid=mozclient&ie=utf-8&oe=utf-8&q=%2B' + crit + '&nocalc=1';
		        break;
		
			case "doUrlNav":
				searchURL = myGooglebarUtil.getElement("ggb_search_criteria").value;
				break;
				
		    default:	        
		        searchURL = 'http://www.google.' + cntry2Srch + '/search?sourceid=mozclient&ie=utf-8&oe=utf-8&q=';
		        searchURL += crit;
		    	searchURL += globalCrit;
		        break;
	    }
	
	    if (myGooglebarPrefUtil.getPrefValue(GB_PREF_RESULT_DATE_SORT_NAME)) {
	    	searchURL = searchURL.replace('sourceid=mozclient','sourceid=mozclient&scoring=d');
	    }
		if (myGooglebarPrefUtil.getPrefValue(GB_PREF_RESULT_MAX_NAME) > 0) {
	    	searchURL = searchURL.replace('sourceid=mozclient','sourceid=mozclient&num=' + myGooglebarPrefUtil.getPrefValue(GB_PREF_RESULT_MAX_NAME));
	    }
	
		// myGooglebarUtil.logMessage(searchURL);
	    this.doAction(searchURL, this.getLocFromEvent(aEvent));
	},

	doSelectionSearch: function(aEvent, aSearchType, aSearchParams) {
	    var critBox = myGooglebarUtil.getElement("ggb_search_criteria");
		var text = myGooglebarUtil.getSelectedText();
	
	    // if we have found some selected text then set the criteria
	    // box to that value and reset the search term buttons,
	    // otherwise continue on with an empty search
	    
	    if (text) {
	        // if there was no previous text or the shift key was pressed
	        // then change the value to the selected text, otherwise append
	        // the text to the end of what was already there.  after that
	        // then the search term buttons need to be updated
	
	    	if (!critBox.value){
				critBox.value = text;
	        }
	        else {
		        if (aEvent.shiftKey) {
					myGooglebarPrefUtil.getPrefValue(GB_PREF_SELECTION_APPENDS_NAME) ? critBox.value = text : critBox.value += " " + text;
		        }
		        else {
					myGooglebarPrefUtil.getPrefValue(GB_PREF_SELECTION_APPENDS_NAME) ? critBox.value += " " + text : critBox.value = text;
		        }
	        }

			// New text in box, so update the buttons...
	        this.refreshSearchButtons();
	        
	        // Do search, if not just appending
	        if (myGooglebarPrefUtil.getPrefValue(GB_PREF_SELECTION_APPENDS_NAME) == aEvent.shiftKey) {
				// Just sneak in a pref change for this one search and reset it when done.
				var savedPref = myGooglebarPrefUtil.getPrefValue(GB_PREF_LOC_NONE_NAME);
		        myGooglebarPrefUtil.setPrefValue(GB_PREF_LOC_NONE_NAME, "1");  // New Tab as default
		        this.doSearch(aEvent, aSearchType, aSearchParams);
		        myGooglebarPrefUtil.setPrefValue(GB_PREF_LOC_NONE_NAME, savedPref);
		    }
		}
	    else {
	        if (myGooglebarPrefUtil.getPrefValue(GB_PREF_SELECTION_APPENDS_NAME) == aEvent.shiftKey) {
		        this.doSearch(aEvent, aSearchType, aSearchParams);
		    }
		} 
	},

	getActFromEvent: function(aEvent) {
	    if (aEvent.altKey) {
	        if (aEvent.ctrlKey) {
	            return GB_DECK_ACTIONS[myGooglebarPrefUtil.getPrefValue(GB_PREF_ACTION_ALT_CTRL_NAME)];
	        }
	        else if (aEvent.shiftKey) {
	            return GB_DECK_ACTIONS[myGooglebarPrefUtil.getPrefValue(GB_PREF_ACTION_ALT_SHIFT_NAME)];
	        }
	        else {
	            return GB_DECK_ACTIONS[myGooglebarPrefUtil.getPrefValue(GB_PREF_ACTION_ALT_NAME)];
	        }
	    }
	    else if (aEvent.ctrlKey) {
	        if (aEvent.shiftKey) {
	            return GB_DECK_ACTIONS[myGooglebarPrefUtil.getPrefValue(GB_PREF_ACTION_CTRL_SHIFT_NAME)];
	        }
	        else {
	            return GB_DECK_ACTIONS[myGooglebarPrefUtil.getPrefValue(GB_PREF_ACTION_CTRL_NAME)];
	        }
	    }
	    else if (aEvent.shiftKey) {
	        return GB_DECK_ACTIONS[myGooglebarPrefUtil.getPrefValue(GB_PREF_ACTION_SHIFT_NAME)];
	    }
	    else {
	        return GB_DECK_ACTIONS[myGooglebarPrefUtil.getPrefValue(GB_PREF_ACTION_NONE_NAME)];
	    }
	},

	getCustomList: function() {
		var customList = new Array();
		
		var custPopup = myGooglebarUtil.getElement('ggb_cust_popup');
		for (var i = custPopup.childNodes.length - 1; i >= 0; i--) {
			customList.push(custPopup.childNodes[i].getAttribute('id'));
		}
		
		return customList;
	},

	getInfoLoc: function() {
	    var win = window._content.document;
	    var location;
	
	    if (/^\s*http:\/\/[^\.]*\.google\..*\/[^\?]*\?.*(q=link:|q=related:|q=site:|u=)([^\+]*)([^&]*)(&.*)*/i.test(win.location.href)) {
	        location = RegExp.$2;
	    }
	    else if (/^\s*http:\/\/.*\/[^\?]*\?.*q=cache:([^\+]*)([^&]*)(&.*)*/i.test(win.location.href)) {
	        location = RegExp.$1;
	    }
	    else {
	        location = win.location;
	    }
	    
	    return escape(location);
	},

	getLocFromEvent: function(aEvent) {
	    // alert("ALT : " + aEvent.altKey + "\nCTRL : " + aEvent.ctrlKey + "\nSHIFT : " + aEvent.shiftKey + "\n");
	    if (aEvent.button == 1) {
	        return "New Tab";
	    }
	    if (aEvent.altKey) {
	        if (aEvent.ctrlKey) {
	            return GB_DECK_LOCS[myGooglebarPrefUtil.getPrefValue(GB_PREF_LOC_ALT_CTRL_NAME)];
	        }
	        else if (aEvent.shiftKey) {
	            return GB_DECK_LOCS[myGooglebarPrefUtil.getPrefValue(GB_PREF_LOC_ALT_SHIFT_NAME)];
	        }
	        else {
	            return GB_DECK_LOCS[myGooglebarPrefUtil.getPrefValue(GB_PREF_LOC_ALT_NAME)];
	        }
	    }
	    else if (aEvent.ctrlKey) {
	        if (aEvent.shiftKey) {
	            return GB_DECK_LOCS[myGooglebarPrefUtil.getPrefValue(GB_PREF_LOC_CTRL_SHIFT_NAME)];
	        }
	        else {
	            return GB_DECK_LOCS[myGooglebarPrefUtil.getPrefValue(GB_PREF_LOC_CTRL_NAME)];
	        }
	    }
	    else if (aEvent.shiftKey) {
	        return GB_DECK_LOCS[myGooglebarPrefUtil.getPrefValue(GB_PREF_LOC_SHIFT_NAME)];
	    }
	    else {
	        return GB_DECK_LOCS[myGooglebarPrefUtil.getPrefValue(GB_PREF_LOC_NONE_NAME)];
	    }
	},

	getNewsLoc: function() {
		var cntryNewsSite = this.mCntryNewsSites[myGooglebarPrefUtil.getPrefValue(GB_PREF_COUNTRY_NEWS_SEARH_NAME)];
		var location = 'http://news.google.' + GB_USA_ONLY_DOMAIN + cntryNewsSite;
	    return location;
	},

	getNewsSearchLoc: function() {
		var cntryNewsSrch = this.mCntryNewsSfxs[myGooglebarPrefUtil.getPrefValue(GB_PREF_COUNTRY_NEWS_SEARH_NAME)];
		var location = 'http://news.google.' + GB_USA_ONLY_DOMAIN + cntryNewsSrch;
		return location;
	},

	hideItem: function(aEvent) {
	    var item = aEvent.target;
	    var ggbItem = item.getAttribute("googlebaritem");
	    var ggbButton, ggbMenuitem;
	
	    ggbButton = myGooglebarUtil.getElement(ggbItem);    
	    ggbButton.setAttribute("hidden", !item.getAttribute("checked"));
	
	    if (myGooglebarPrefUtil.getPrefValue(GB_PREF_HIDE_MENU_NAME) && ggbItem.indexOf("_separate") > 0) {
	        ggbItem = ggbItem.replace("_separate", "");
	        ggbMenuitem = myGooglebarUtil.getElement(ggbItem);
	
	        if (ggbMenuitem) {
	            ggbMenuitem.setAttribute("hidden", item.getAttribute("checked"));
	        }
	    }
	
		if (myGooglebarUtil.isInPrefs()) {
			this.syncAllMainPopups();
			// this.ordinals();
		}
	
		myGooglebarDnD.updateSeparators();
	    this.refreshSearchButtons();
	},

	initialize: function() {	
	    if (!this.mToolbarLoaded) {
	        this.mToolbarLoaded = true;
			this.initOnce();
	    }

	    this.setUniv(myGooglebarPrefUtil.getPrefValue(GB_PREF_UNIVERSITY_ID_NAME), myGooglebarPrefUtil.getPrefValue(GB_PREF_UNIVERSITY_LABEL_NAME));
	
		myGooglebarUtil.getElement("ggb_ctxt_tbar_LabelsButtonItem").setAttribute("checked", myGooglebarPrefUtil.getPrefValue(GB_PREF_BUTTON_LABELS_NAME));
		this.labels();
	
		myGooglebarUtil.getElement("ggb_ctxt_tbar_OrderLockButtonItem").setAttribute("checked", myGooglebarPrefUtil.getPrefValue(GB_PREF_LOCK_ORDINALS_NAME));
		this.ordinals();
		
		this.syncAllMainPopups();
		this.updateContextMenu();
		this.updateUp();
	
		/*	
	    if (myGooglebarUtil.isInFirefox()) {
			// workaround bug 6171
		    try {
		        if(myGooglebarPrefUtil.getPrefValue(GB_PREF_HISTORY_SAVE_NAME) == "false") {
		        	myGooglebarPrefUtil.setPrefValue(GB_PREF_HISTORY_MAX_NAME, 0);
		        }
		    }
		    catch(e) {
		    	myGooglebarUtil.logMessage(e);
		    }
			// end workaround bug 6171
	    }
	 	*/
	},

	initCustom: function() {
		var i;
		
	    for (i = 0; myGooglebarUtil.getProperty("gb.country." + i) != "endoflist"; i++) {
	        this.mCntrySfxs[i] = myGooglebarUtil.getProperty("gb.country." + i);
	    }
	
	    for (i = 0; myGooglebarUtil.getProperty("gb.mycountry." + i) != "endoflist"; i++) {
	        this.mMyCntrySfxs[i] = myGooglebarUtil.getProperty("gb.mycountry." + i);
	    }
	
	    for (i = 0; myGooglebarUtil.getProperty("gb.mylang." + i) != "endoflist"; i++) {
	        this.mMyLangSfxs[i] = myGooglebarUtil.getProperty("gb.mylang." + i);
	    }
	
	    for (i = 0; myGooglebarUtil.getProperty("gb.news.country." + i) != "endoflist"; i++) {
	        this.mCntryNewsSfxs[i] = myGooglebarUtil.getProperty("gb.news.country." + i);
	        this.mCntryNewsSites[i] = myGooglebarUtil.getProperty("gb.news.country.site." + i);
	    }

	    for (i = 0; myGooglebarUtil.getProperty("gb.maps.country." + i) != "endoflist"; i++) {
	        this.mCntryMapsSfxs[i] = myGooglebarUtil.getProperty("gb.maps.country." + i);
	    }

	    for (i = 0; myGooglebarUtil.getProperty("gb.froogle.country." + i) != "endoflist"; i++) {
	        this.mCntryFroogleSfxs[i] = myGooglebarUtil.getProperty("gb.froogle.country." + i);
	    }

	    var customArray = myGooglebarPrefUtil.getPrefValue(GB_PREF_CUSTOM_LIST_NAME);
		this.updateCustomList(customArray);
	},

	initOnce: function() {
        if (!myGooglebarUtil.isInFirefox()){
        	myGooglebarInstallUtil.isNewInstall();
        }

		this.register();

        var hideExtras = !(myGooglebarUtil.getElement("ggb_extras_box").childNodes.length > 0);
        myGooglebarUtil.getElement("ggb_extras_box").setAttribute("hidden", hideExtras);
        myGooglebarUtil.getElement("ggb_ctxt_tbar_ExtrasButtonItem").setAttribute("hidden", hideExtras);

		myGooglebarUtil.getElement("ggb_ctxt_tbar_LabelsButtonItem").addEventListener("DOMAttrModified", myGooglebar.listenLabelAttr, false);
		myGooglebarUtil.getElement("ggb_ctxt_tbar_OrderLockButtonItem").addEventListener("DOMAttrModified", myGooglebar.listenOrdinalAttr, false);

		/*
		if (myGooglebarUtil.isInFirefox()){
        	myGooglebarUtil.getElement("ggb_prefill_separate").setAttribute("hidden", true);
        	myGooglebarUtil.getElement("ggb_ctxt_tbar_PrefillButtonItem").setAttribute("hidden", true);
		}
		*/
		
        this.syncMenuButton("ggb_CompMenu", GB_PREF_LAST_SEARCH_COMP_NAME);
        this.syncMenuButton("ggb_CustMenu", GB_PREF_LAST_SEARCH_CUST_NAME);
        this.syncMenuButton("ggb_InfoMenu", GB_PREF_LAST_SEARCH_INFO_NAME);
        this.syncMenuButton("ggb_SpecMenu", GB_PREF_LAST_SEARCH_SPEC_NAME);
        this.syncMenuButton("ggb_SrchMenu", GB_PREF_LAST_SEARCH_SRCH_NAME);

        myGooglebarDnD.initCustomOrder();
        this.initCustom();
		this.initToolbarMenu();
		this.updateDesktopItems();

		myGooglebarMail.initialize();

		myGooglebarUtil.logMessage('Initialized googlebar');
	},

	initToolbarMenu: function() {
		// Try to put the hide/show menuitem under the toolbar popup
		var toolbarContainer = myGooglebarUtil.getElement("menu_viewPopup");
		var checkMenuitem = myGooglebarUtil.getElement("googlebar").getAttribute("hidden") != "true";
		myGooglebarUtil.getElement("ggb_ctxt_brwsr_ShowGooglebarItem").setAttribute("checked", checkMenuitem);
	    			
	    if (toolbarContainer != null) {
	    	var toolbarPopup;
	    
		    for (var i = 0; i < toolbarContainer.childNodes.length; i++) {
		        if (toolbarContainer.childNodes.item(i).nodeName == "menu" && toolbarContainer.childNodes.item(i).getAttribute("label") == "Toolbars") {
		        	toolbarPopup = toolbarContainer.childNodes.item(i).getElementsByTagName("menupopup").item(0);
		       	}
		    }
		    
		    var tempNode = document.createElement("menuitem");
		    tempNode.setAttribute("id", "ggb_view_googlebar_menuitem");
            tempNode.setAttribute("label", "Googlebar");
            tempNode.setAttribute("class", "menuitem-iconic");
            tempNode.setAttribute("type", "checkbox");
            tempNode.setAttribute("key", "ggb_key_ShowHideAccel");
            tempNode.setAttribute("observes", "ggb_cmd_ShowHide");
            tempNode.setAttribute("accesskey", "G");
		    
		    if (toolbarPopup != null) {
		    	toolbarPopup.appendChild(document.createElement("menuseparator"));
		    	toolbarPopup.appendChild(tempNode);
		   	}
		    else {
		    	toolbarContainer.appendChild(tempNode);
		    }
		    
		    
			tempNode.setAttribute("checked", checkMenuitem);
	    }
	},

	labels: function() {
	    if (myGooglebarPrefUtil.getPrefValue(GB_PREF_BUTTON_LABELS_NAME)) {
	        this.labelsOn();
	    }
	    else {
	        this.labelsOff();
	    }
	},
	
	labelsOff: function() {
	    var container = myGooglebarUtil.getElement("ggb_toolbar_buttons");
	    var containerButtons = container.getElementsByTagName("toolbarbutton");
	
	    for (var i = 0; i < containerButtons.length; i++) {
	    	if (!containerButtons[i].getAttribute("label_ext")) {
	            containerButtons[i].removeAttribute("label");
	        }
	        else {
	        	containerButtons[i].setAttribute("label", containerButtons[i].getAttribute("label_ext"));
	        }
	        myGooglebarUtil.updateClass(containerButtons[i], 0, containerButtons[i].getAttribute("buttonclass"));
	    }
	},

	labelsOn: function() {
	    var container = myGooglebarUtil.getElement("ggb_toolbar_buttons");
	    var containerButtons = container.getElementsByTagName("toolbarbutton");
	
	    for (var i = 0; i < containerButtons.length; i++) {
	    	if (!containerButtons[i].getAttribute("label_ext")) {
	            containerButtons[i].setAttribute("label", containerButtons[i].getAttribute("buttonlabel"));
	        }
	        else {
	        	var label = containerButtons[i].getAttribute("buttonlabel") + " " + containerButtons[i].getAttribute("label_ext");
	        	containerButtons[i].setAttribute("label", label);
	        }
        
	        myGooglebarUtil.updateClass(containerButtons[i], 0, "ggb_css_button_labeled");
	    }
	},

	listenLabelAttr: function(aEvent) {
		if (aEvent && aEvent.attrName == 'checked') {
			// Actual action should now happen in the pref observer when the pref is updated
			myGooglebarPrefUtil.setPrefValue(GB_PREF_BUTTON_LABELS_NAME, aEvent.newValue == "true");
		}
	},
	
	listenOrdinalAttr: function(aEvent) {
		if (aEvent && aEvent.attrName == 'checked') {
			// Actual action now happen in the pref observer when the pref is updated
			myGooglebarPrefUtil.setPrefValue(GB_PREF_LOCK_ORDINALS_NAME, aEvent.newValue == "true");
		}
	},

	lookForURL: function() {
		var criteria = myGooglebarUtil.getElement("ggb_search_criteria");
		var testIndx;
		var tempval  = criteria.value;
	
		try {
			testIndx = tempval.search(/^(http|https|ftp|file|irc|mailto|news|nntp):/i);
		}
		catch (e) {
			testIndx = -1;
		}
	
		if (testIndx != -1) {
			return 1;
		}
		else {
			try {
				testIndx = tempval.search(/^(www\.)/i);
			}
			catch (e) {
				// replace the prefix to http://
				testIndx = -1;
			}
		}
		    
		if (testIndx != -1) {
	    	var newval = "http://" + criteria.value;
	    	criteria.value = newval;
			return 1;
		}
		else {
			// we didn't find it, or there was a general error	
			return 0;
		}	
	},

	openGooglePrefs: function() {
	    var cntry2Srch = this.mCntrySfxs[myGooglebarPrefUtil.getPrefValue(GB_PREF_COUNTRY_SEARH_NAME)];
	    window._content.document.location = 'http://www.google.' + cntry2Srch + '/preferences';
	},
	
	openPrefs: function() {
	    if (myGooglebarUtil.isInFirefox()) {
	        window.openDialog('chrome://googlebar/content/googlebarPrefDialog.xul', 'Googlebar Options', 'chrome');
	    }
	    else {
	        goPreferences('googlebarItem', 'chrome://googlebar/content/pref/pref-googlebar1.xul', 'googlebarItem');
	    }
	},
	
	ordinals: function() {
		if (myGooglebarPrefUtil.getPrefValue(GB_PREF_LOCK_ORDINALS_NAME)) {
			myGooglebarDnD.lockOrdinalOrder();
		}
		else {
			myGooglebarDnD.unlockOrdinalOrder();
		}
	},
	
	populateUniv: function() {
		if (window._content.document.location.href != GB_UNIV_URL) {
			return;
		}
	
	    var univID = "ggb_univ_popup";
	    var univMenu;
	    var univPopup;
	
	    // Remove old instances
	    var univContainer = myGooglebarUtil.getElement(univID);
	    for (var i = 3; i < univContainer.childNodes.length; i++) {
	        univMenu = univContainer.childNodes.item(i);
	        univPopup = univMenu.childNodes.item(0);
	        for (var j = univPopup.childNodes.length - 1; j >= 0; j--) {
	            univPopup.removeChild(univPopup.childNodes.item(j));
	        }
	    }
	
	    var doc = window._content.document.documentElement;
	    var aElems = doc.getElementsByTagName("A");
	    var aData;
	    var aHref;
	    var startsWith;
	    var tempNode;
	
	    for (var k = 0; k < aElems.length; k++) {
	        aHref = aElems[k].getAttribute("href");
	        if (aHref) {
	            if (/^\s*http:\/\/www.google.com\/univ\/(.*)/i.test(aHref)) {
	                aData = aElems[k].firstChild.data;
	                aData = aData.replace(/\s+/g, " ");
	                // aData = data.replace(/[\t\n\r ]+/g, " ");
	                startsWith = aData.substring(0, 1).toLowerCase();
	
	                univPopup = myGooglebarUtil.getElement(univID + "_" + startsWith);
	
	                if (univPopup) {
	                    tempNode = document.createElement("menuitem");
	                    tempNode.setAttribute("label", aData);
	                    tempNode.setAttribute("class", "ggb_css_menuitem_all");
	                    tempNode.setAttribute("oncommand", "myGooglebar.setUniv('" + aHref + "', '" + aData + "'); event.preventBubble();");
	                    tempNode.setAttribute("tooltiptext", aHref);
	                    univPopup.appendChild(tempNode);
	                }
	            }
	        }
	    }
	    alert(myGooglebarUtil.getProperty("gb.button.univ.done"));
	},
	
	refreshSearchButtons: function() {
		this.clearTerms();
		
		var critBox = myGooglebarUtil.getElement("ggb_search_criteria");
		this.addTerms(critBox.value);
	},
	
	register: function() {
		window.getBrowser().addProgressListener(myGooglebarLocListener, Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
		window.getBrowser().addProgressListener(myGooglebarLocListener, Components.interfaces.nsIWebProgress.STATE_STOP);
	},
	
	resetParent: function(aMenuButton) {
	    aMenuButton.removeAttribute("oncommand");
	    aMenuButton.setAttribute("oncommand", aMenuButton.getAttribute('originalcommand'));
	
	    aMenuButton.setAttribute("buttonclass", aMenuButton.getAttribute('originalclass'));
	    aMenuButton.setAttribute("class", aMenuButton.getAttribute('originalclass') + " ggb_css_drag_border_none ggb_css_toolbarbutton");
	
	    aMenuButton.setAttribute("buttonlabel", aMenuButton.getAttribute('originallabel'));
	    aMenuButton.setAttribute("tooltiptext", aMenuButton.getAttribute('originaltooltip'));
	    
	    if (aMenuButton.getAttribute("label")) {
	        aMenuButton.setAttribute("label", aMenuButton.getAttribute('originallabel'));
	    }
	},

	resetParents: function() {
		if (myGooglebarPrefUtil.getPrefValue(GB_PREF_SAVE_LAST_SEARCH_NAME))
			return;
			
		myGooglebarPrefUtil.setPrefValue(GB_PREF_LAST_SEARCH_COMP_NAME, GB_PREF_LAST_SEARCH_COMP_VALUE);
		myGooglebarPrefUtil.setPrefValue(GB_PREF_LAST_SEARCH_CUST_NAME, GB_PREF_LAST_SEARCH_CUST_VALUE);
		myGooglebarPrefUtil.setPrefValue(GB_PREF_LAST_SEARCH_INFO_NAME, GB_PREF_LAST_SEARCH_INFO_VALUE);
		myGooglebarPrefUtil.setPrefValue(GB_PREF_LAST_SEARCH_SPEC_NAME, GB_PREF_LAST_SEARCH_SPEC_VALUE);
		myGooglebarPrefUtil.setPrefValue(GB_PREF_LAST_SEARCH_SRCH_NAME, GB_PREF_LAST_SEARCH_SRCH_VALUE);

		this.resetParent(myGooglebarUtil.getElement('ggb_CompMenu'));
		this.resetParent(myGooglebarUtil.getElement('ggb_CustMenu'));
		this.resetParent(myGooglebarUtil.getElement('ggb_InfoMenu'));
		this.resetParent(myGooglebarUtil.getElement('ggb_SpecMenu'));
		this.resetParent(myGooglebarUtil.getElement('ggb_SrchMenu'));
	},

	searchTermsToggle: function() {
	    var ggbsrchterms = myGooglebarUtil.getElement("ggb_searchterms_box");
	    var searchTermMenu = myGooglebarUtil.getElement("ggb_searchterms_menu");
	    ggbsrchterms.setAttribute("hidden", !ggbsrchterms.hidden );
	
	    // when we are overflowing, set the hidden status of the overflow
	    // menu accordingly
	    if (ggbsrchterms.getAttribute("googlebaroverflow")=="true") {
	        searchTermMenu.setAttribute("hidden", ggbsrchterms.hidden );
	    }
	
	    // when the search buttons were unhidden, we have to check
	    // if they fit
	    if (!ggbsrchterms.hidden) {
	        this.refreshSearchButtons();
	    }
	    
	    return true;
	},
	
	setUniv: function(aURL, aName) {
	    var univBtn;
	
	    univBtn = myGooglebarUtil.getElement("ggb_UnivButton");
	    univBtn.removeAttribute("oncommand");
	    univBtn.setAttribute("onclick", "if (event.button == 1) { myGooglebar.univGo(event, '" + aURL + "') }");
	    univBtn.setAttribute("oncommand", "myGooglebar.univGo(event, '" + aURL + "');");
	    univBtn.setAttribute("tooltiptext", myGooglebarUtil.getProperty("gb.button.univ") + " - " + aName);
	
	    myGooglebarPrefUtil.setPrefValue(GB_PREF_UNIVERSITY_ID_NAME, aURL);
	    myGooglebarPrefUtil.setPrefValue(GB_PREF_UNIVERSITY_LABEL_NAME, aName);
	},

	setVisible: function(aBoolean) {
		var googlebar = myGooglebarUtil.getElement( "googlebar" );
		var currCrit = myGooglebarUtil.getElement("ggb_search_criteria").value ;

		// Show if hidden
		if (aBoolean) {
			// If you have some text in the criteria box, updating searchword buttons
			// on reappearance is unsightly. The buttons adjust one character at a time.
		
			if(!googlebar.getAttribute("hidden") || googlebar.getAttribute("hidden") == "false")
				return;
		
			this.clearCriteria();
			this.clearTerms();
			goToggleToolbar("googlebar", "ggb_cmd_ShowHide");
			setTimeout("myGooglebarUtil.getElement('ggb_search_criteria').focus()", 100);
		
			// let's just blank it out... for now.
			currCrit = "";
			myGooglebarUtil.getElement("ggb_search_criteria").value = currCrit;
			this.addTerms(currCrit);
		}
		// Hide if shown
		else {
			if(googlebar.getAttribute("hidden") && googlebar.getAttribute("hidden") == "true")
				return;
		
			goToggleToolbar("googlebar", "ggb_cmd_ShowHide");
			setTimeout("window.document.getElementById('urlbar').focus()", 0);
		}
	},

	showMenuPopup: function(aEvent) {
	    if ((aEvent.target.getAttribute("type") == "menu" || aEvent.target.getAttribute("type") == "menu-button") && aEvent.target.firstChild) {
	      aEvent.target.firstChild.showPopup();
	    }
	},

	syncAllMainPopups: function() {
	    var ggbPopupInUse = myGooglebarUtil.getElement("ggb_toolbar_buttons");
	
		this.syncMainPopup("ggb_CustMenu", "ggb_ctxt_tbar_CustomMenuItem");
	    this.syncMainPopup("ggb_search_separate", "ggb_ctxt_tbar_SearchButtonItem");
	    this.syncMainPopup("ggb_site_separate", "ggb_ctxt_tbar_SiteButtonItem");
	    this.syncMainPopup("ggb_lucky_separate", "ggb_ctxt_tbar_LuckyButtonItem");
	    this.syncMainPopup("ggb_images_separate", "ggb_ctxt_tbar_ImagesButtonItem");
	    this.syncMainPopup("ggb_video_separate", "ggb_ctxt_tbar_VideoButtonItem");
	    this.syncMainPopup("ggb_groups_separate", "ggb_ctxt_tbar_GroupsButtonItem");
	    this.syncMainPopup("ggb_directory_separate", "ggb_ctxt_tbar_DirectoryButtonItem");
	    this.syncMainPopup("ggb_news_separate", "ggb_ctxt_tbar_NewsButtonItem");
	    this.syncMainPopup("ggb_answers_separate", "ggb_ctxt_tbar_AnswersButtonItem");
	    this.syncMainPopup("ggb_scholar_separate", "ggb_ctxt_tbar_ScholarButtonItem");
	    this.syncMainPopup("ggb_print_separate", "ggb_ctxt_tbar_PrintButtonItem");
   	    this.syncMainPopup("ggb_desktop_separate", "ggb_ctxt_tbar_DesktopButtonItem");
	    
	    this.syncMainPopup("ggb_options_separate", "ggb_ctxt_tbar_OptionsButtonItem");
	    this.syncMainPopup("ggb_catalog_separate", "ggb_ctxt_tbar_CatalogButtonItem");
	    this.syncMainPopup("ggb_froogle_separate", "ggb_ctxt_tbar_FroogleButtonItem");
	    this.syncMainPopup("ggb_stocks_separate", "ggb_ctxt_tbar_StocksButtonItem");
	    this.syncMainPopup("ggb_unclesam_separate", "ggb_ctxt_tbar_UnclesamButtonItem");
	    this.syncMainPopup("ggb_dictionary_separate", "ggb_ctxt_tbar_DictionaryButtonItem");
	    this.syncMainPopup("ggb_glossary_separate", "ggb_ctxt_tbar_GlossaryButtonItem");
	    this.syncMainPopup("ggb_nocalc_separate", "ggb_ctxt_tbar_NoCalcButtonItem");

	    this.syncMainPopup("ggb_blogger_separate", "ggb_ctxt_tbar_BloggerButtonItem");
	    this.syncMainPopup("ggb_maps_separate", "ggb_ctxt_tbar_MapsButtonItem");
	    this.syncMainPopup("ggb_mycountry_separate", "ggb_ctxt_tbar_MyCountryButtonItem");
	    this.syncMainPopup("ggb_mylocal_separate", "ggb_ctxt_tbar_MyLocalButtonItem");

	    this.syncMainPopup("ggb_bsd_separate", "ggb_ctxt_tbar_BsdButtonItem");
	    this.syncMainPopup("ggb_linux_separate", "ggb_ctxt_tbar_LinuxButtonItem");
	    this.syncMainPopup("ggb_mac_separate", "ggb_ctxt_tbar_MacButtonItem");
	    this.syncMainPopup("ggb_microsoft_separate", "ggb_ctxt_tbar_MicrosoftButtonItem");
	
	    this.syncMainPopup("ggb_cached_separate", "ggb_ctxt_tbar_CachedButtonItem");
	    this.syncMainPopup("ggb_similar_separate", "ggb_ctxt_tbar_SimilarButtonItem");
	    this.syncMainPopup("ggb_backwards_separate", "ggb_ctxt_tbar_BackwardsButtonItem");
	    this.syncMainPopup("ggb_translate_separate", "ggb_ctxt_tbar_TranslateButtonItem");
	
		this.syncMainPopup("ggb_SrchMenu", "ggb_ctxt_tbar_SrchButtonItem");
	    this.syncMainPopup("ggb_SpecMenu", "ggb_ctxt_tbar_SpecButtonItem");
	    this.syncMainPopup("ggb_CompMenu", "ggb_ctxt_tbar_CompButtonItem");
	    this.syncMainPopup("ggb_UnivButton", "ggb_ctxt_tbar_UnivButtonItem");
	    this.syncMainPopup("ggb_extras_box", "ggb_ctxt_tbar_ExtrasButtonItem");
	    this.syncMainPopup("ggb_InfoMenu", "ggb_ctxt_tbar_InfoButtonItem");
	    this.syncMainPopup("ggb_UpDirButton", "ggb_ctxt_tbar_UpDirButtonItem");
	    this.syncMainPopup("ggb_navigation_box", "ggb_ctxt_tbar_NavButtonItem");
	    this.syncMainPopup("ggb_highlightplus_button", "ggb_ctxt_tbar_HighlightPlusButtonItem");
	    this.syncMainPopup("ggb_highlight_button", "ggb_ctxt_tbar_HighlightButtonItem");
	    this.syncMainPopup("ggb_searchterms_box", "ggb_ctxt_tbar_SearchTermsButtonItem");
	},
	
	syncMainPopup: function(src, dest) {
	    var ggbButtonState = myGooglebarUtil.getElement(src);
	    var ggbPopupChecked = myGooglebarUtil.getElement(dest);

		if (!ggbPopupChecked) {
			myGooglebarUtil.logMessage("Unable to sync " + dest + ", element not found");
			return;
		}
		
	    ggbPopupChecked.setAttribute("checked", !(ggbButtonState.getAttribute("hidden") == "true"));
	
	    if (src.indexOf("_separate") > 0) {
	        var ggbItem = src.replace("_separate", "");
	        var ggbMenuitem = myGooglebarUtil.getElement(ggbItem);
	
	        if (ggbMenuitem) {
	        	if (myGooglebarPrefUtil.getPrefValue(GB_PREF_HIDE_MENU_NAME)) {
	            	ggbMenuitem.setAttribute("hidden", !(ggbButtonState.getAttribute("hidden") == "true"));
	            }
	            else {
		            ggbMenuitem.setAttribute("hidden", false);
	            }
	        }
	    }
	},

	syncMenuButton: function(aButton, aOption) {
	    // If there was a previous search selected for the
	    // menu button and the preference is set to save
	    // these settings then attempt to restore the
	    // selection, if possible, otherwise just ignore it
	
	    if (!myGooglebarPrefUtil.getPrefValue(GB_PREF_SAVE_LAST_SEARCH_NAME)) {
	        return;
	    }
	
	    var optionValue;
	
	    try {
	        optionValue = myGooglebarPrefUtil.getPrefValue(aOption);
	    }
	    catch(e) {
	    	myGooglebarUtil.logMessage(  "\n" + e + "\n" );
	        myGooglebarPrefUtil.setPrefValue(aOption, "");
	        optionValue = "";
	    }
	
	    var menuButton = myGooglebarUtil.getElement(aButton);
	    var separateButton = myGooglebarUtil.getElement(optionValue + "_separate");
	
	    if (!menuButton || !separateButton) {
	        return;
	    }
	
	    menuButton.removeAttribute("oncommand");
	    menuButton.setAttribute("oncommand", separateButton.getAttribute("oncommand"));
	
	    menuButton.setAttribute("buttonclass", separateButton.getAttribute("buttonclass"));
	    menuButton.setAttribute("class", separateButton.getAttribute("class"));
	
		menuButton.setAttribute("buttonlabel", separateButton.getAttribute("buttonlabel"));
		menuButton.setAttribute("tooltiptext", separateButton.getAttribute("tooltiptext"));
	
	    if (separateButton.getAttribute("label")) {
	        menuButton.setAttribute("label", separateButton.getAttribute("label"));
	    }
	},
	
	toggleDisplay: function() {
		if (myGooglebarUtil.getElement('googlebar').getAttribute('hidden') == 'true') {
			myGooglebar.setVisible(true);
		}
		else {
			myGooglebar.setVisible(false);
		}
	},
	
	univGo: function(aEvent) {
	    if (!myGooglebarPrefUtil.getPrefValue(GB_PREF_UNIVERSITY_ID_NAME)) {
	        alert(myGooglebarUtil.getProperty("gb.button.univ.none"));
	    }
	    else {
	        this.doSearch(aEvent, "university");
	    }
	},
	
	unregister: function() {
		window.getBrowser().removeProgressListener(myGooglebarLocListener);
	},

	updateContextMenu: function() {	
	    var cntxtmnu = myGooglebarUtil.getElement('ggb_ctxt_brwsr_menu');
	    var cntxtmnu_sep = myGooglebarUtil.getElement('ggb_ctxt_brwsr_separator');
	    var cntxtmnuopt = !myGooglebarPrefUtil.getPrefValue(GB_PREF_CONTEXT_MENU_NAME);
	
	   	cntxtmnu.setAttribute('hidden', cntxtmnuopt);
		cntxtmnu_sep.setAttribute('hidden', cntxtmnuopt);
	},
	
	updateCriteriaDropmarker: function() {
		var critMenu = myGooglebarUtil.getElement("ggb_search_criteria");
	
	    if (!critMenu) {
	    	// Removed extraneous log messages, only useful for debugging
			// myGooglebarUtil.logMessage(myGooglebarUtil.getProperty("gb.googlebarUpdateCriteriaDropmarker.1"));
	    }
	    else {
	        var critMenuDrop = myGooglebarUtil.getAnonymousElement(critMenu, "class", "menulist-dropmarker");
	        
	        if (!critMenuDrop) {
		        // Removed extraneous log messages, only useful for debugging
	        	// myGooglebarUtil.logMessage(myGooglebarUtil.getProperty("gb.googlebarUpdateCriteriaDropmarker.2"));
	        }
	        else {
		        critMenuDrop.setAttribute("hidden", !myGooglebarPrefUtil.getPrefValue(GB_PREF_HISTORY_SAVE_NAME));
		    }
	        
	    }
	},
		
	updateCustomList: function(aCustomList) {
		if (!aCustomList)
			aCustomList = new Array();
	
		var custPopup = myGooglebarUtil.getElement('ggb_cust_popup');
		var i;
		
		for (i = custPopup.childNodes.length - 1; i >= 0; i--) {
			custPopup.removeChild(custPopup.childNodes[i]);
		}
	
		var customArray = new Array(aCustomList.length);
			
		for (i = 0; i < aCustomList.length; i++) {
			var custChild = myGooglebarUtil.getElement(aCustomList[i]).cloneNode(false);
			custPopup.appendChild(custChild);
			customArray[i] = custChild.getAttribute('id');
		}
		
	    myGooglebarPrefUtil.setPrefValue(GB_PREF_CUSTOM_LIST_NAME, customArray);
	},

	updateDesktopItems: function() {
		var hasURL = myGooglebarPrefUtil.getPrefValue(GB_PREF_DESKTOP_URL_NAME);
		hasURL = (hasURL != null && hasURL != "");  // converting from string to boolean

		var hasID = myGooglebarPrefUtil.getPrefValue(GB_PREF_DESKTOP_ID_NAME);
		hasID = (hasID != null && hasID != "");  // converting from string to boolean
		
		var hasSearchID = myGooglebarPrefUtil.getPrefValue(GB_PREF_DESKTOP_SEARCH_ID_NAME);
		hasSearchID = (hasSearchID != null && hasSearchID != "");  // converting from string to boolean
		
		var disableIt = !(hasURL && hasID && hasSearchID);
		
		myGooglebarUtil.getElement("ggb_desktop").setAttribute("disabled", disableIt);
		myGooglebarUtil.getElement("ggb_desktop_separate").setAttribute("disabled", disableIt);
	},

	updateMaps: function() {
		if (!window._content.document || !myGooglebarPrefUtil.getPrefValue(GB_PREF_SATELLITE_NAME)) {
	        return;
	    }
	
		try {
		    if (/.*:\/\/maps.google.com\/maps.*/.test(window._content.document.location.href)) {
		    	var toggleElem = window._content.document.getElementById("toggle");
		   		var satElem = toggleElem.getElementsByTagName("A")[1];
		    	if (!satElem.getAttribute("style")) {
		    		satElem.onclick();
		    	}
		    }
		}
		catch (e) {
			myGooglebarUtil.logMessage("Problem with Google Maps:\n" + e);
		}
	},

	updateMenuParent: function(aEvent) {
	    if (!myGooglebarPrefUtil.getPrefValue(GB_PREF_SAVE_LAST_SEARCH_NAME)) {
	        return;
	    }
	
	    var menuButton = aEvent.target.parentNode.parentNode;
	    var separateButton = myGooglebarUtil.getElement(aEvent.target.getAttribute("id") + "_separate");
	
	    menuButton.removeAttribute("oncommand");
	    menuButton.setAttribute("oncommand", separateButton.getAttribute("oncommand"));
	
	    menuButton.setAttribute("buttonclass", separateButton.getAttribute("buttonclass"));
	    menuButton.setAttribute("class", separateButton.getAttribute("class"));
	    
	    menuButton.setAttribute("buttonlabel", separateButton.getAttribute("buttonlabel"));
	    menuButton.setAttribute("tooltiptext", separateButton.getAttribute("tooltiptext"));
	
	    if (separateButton.getAttribute("label")) {
	        menuButton.setAttribute("label", separateButton.getAttribute("label"));
	    }
	
	    if (menuButton.getAttribute("id") == "ggb_CompMenu") {
	        myGooglebarPrefUtil.setPrefValue(GB_PREF_LAST_SEARCH_COMP_NAME, aEvent.target.getAttribute("id"));
	    }
	    else if (menuButton.getAttribute("id") == "ggb_CustMenu") {
	        myGooglebarPrefUtil.setPrefValue(GB_PREF_LAST_SEARCH_CUST_NAME, aEvent.target.getAttribute("id"));
	    }
	    else if (menuButton.getAttribute("id") == "ggb_InfoMenu") {
	        myGooglebarPrefUtil.setPrefValue(GB_PREF_LAST_SEARCH_INFO_NAME, aEvent.target.getAttribute("id"));
	    }
	    else if (menuButton.getAttribute("id") == "ggb_SpecMenu") {
	        myGooglebarPrefUtil.setPrefValue(GB_PREF_LAST_SEARCH_SPEC_NAME, aEvent.target.getAttribute("id"));
	    }
	    else if (menuButton.getAttribute("id") == "ggb_SrchMenu") {
	        myGooglebarPrefUtil.setPrefValue(GB_PREF_LAST_SEARCH_SRCH_NAME, aEvent.target.getAttribute("id"));
	    }
	    else {
	        alert(myGooglebarUtil.getProperty("gb.googlebarUpdateParent.1") + menuButton.getAttribute("id")); 
	    }
	},

	updateNavigation: function() {
		var url = window._content.location.href;
	
		// This block of code was put in to update the navigation list when location
		// changes, even when not using the navigation items.  This may or may not
		// be the desired behavior.  To remove this functionality the entire marked
		// block of code can just simply be replaced with a single line:
		//
		//   myGooglebarNavigation.loadResults();
		//
		// That's it.

		// Begin Update Navigation Block
		var inResult = myGooglebarNavigation.checkLocation(url);
		
		if (!inResult) {
        	myGooglebarNavigation.loadResults();
        }
        // End Update Navigation Block
	},

	updateNewWindow: function() {
		// Only need to update from another window once
		// Get out of here if that has already happened
		if (this.mOpenerLoaded)
			return;
			
		// This method is used to update the current search terms 
		// from another window. This should happen when
		// 1. a new window is opened for the first time called from myGooglebar.displayCheck()
		// 2. a google search result
	
		var otherCriteria = null;
		var otherHighlight = null;
		var thisCriteria = myGooglebarUtil.getElement("ggb_search_criteria");
		var foundOther = false;
	
		// If this window was opened from another window, just use
		// the criteria from the opener
		if (window.opener && window.opener.document) {
			// myGooglebarUtil.logMessage("Found another window from opener");
			otherCriteria = window.opener.document.getElementById("ggb_search_criteria");
			otherHighlight = window.opener.document.getElementById("ggb_highlight_button");
	      
			if (otherCriteria){
				// myGooglebarUtil.logMessage("Found another search criteria box from opener");
				thisCriteria.value = otherCriteria.value;
				this.mOpenerLoaded = true;
				foundOther = true;
				myGooglebarUtil.logMessage('Updated criteria from window opener');
			}
		}
		
		if (!foundOther) {
			// window had not opener nor search criteria, so try to get the criteria from somewhere else
	    	var openwindows = GB_WINDOW_SVC.getEnumerator(null);
	    
			// check all open windows for ggb_search_criteria
			// this will always get the search terms of the last window
			// in the list, this might not be the last window which was
			// selected, so this might not always give the expected result
			while (openwindows.hasMoreElements()) {
				var win = openwindows.getNext();
	
				if (win != window) {
					// myGooglebarUtil.logMessage("Found another window from somewhere else");
					otherCriteria = win.document.getElementById("ggb_search_criteria");
					otherHighlight = win.document.getElementById("ggb_highlight_button");
	        
					if (otherCriteria) {
						// myGooglebarUtil.logMessage("Found another search criteria box from somewhere else");
			 			thisCriteria.value = otherCriteria.value;
						this.mOpenerLoaded = true;
						myGooglebarUtil.logMessage('Updated criteria from another window containing a googlebar');
						break;
					}
				}
			}
		}
	
		if (otherHighlight && otherHighlight.getAttribute("activated") == "true") {
			myGooglebarHighlight.highLightInPage("on");
		}
	},
	
	updateUp: function() {
	    if (!window._content.document) {
	        return;
	    }
	
	    var currentAddy = (myGooglebarPrefUtil.getPrefValue(GB_PREF_UP_IGNORE_QUERY_NAME)) ? window._content.document.location.href.split("?")[0] : window._content.document.location.href;
	    var arCurrentAddy = currentAddy.split("/");
	    var curPath = false;
	    var i;
	
	    // Remove old instances
	    var upContainer = myGooglebarUtil.getElement("ggb_up_dir_popup");
	    for (i = upContainer.childNodes.length - 1; i >= 0; i--) {
	        upContainer.removeChild(upContainer.childNodes.item(i));
	    }
	
	    // Add new instances
	    for (i = arCurrentAddy.length - 1; i > 2; i--) {
	        if (arCurrentAddy[i].length) {
	            curPath = myGooglebarUtil.buildURL(arCurrentAddy, i);
	            this.updateUpAppendItem(upContainer, curPath);
	        }
	    }
	
	    // If we are not at the default web host (www) then add that
	    // host to the bottom of the list as Google toolbar does
	    //
	    // e.g. toolbar.google.com has www.google.com listed on "up" popup menu
	
	    if (arCurrentAddy.length >= 3) {
	        var currentHost = arCurrentAddy[2];
	        var arHost = currentHost.split(".");
	        var wwwHost;
	
			// Assumes somthing like this cs.insertschoolhere.edu, up converts to www.insertschoolhere.edu
			// ignores somewhere.co.uk or somewhere.com.au or similar
	        if (arHost.length == 3 && arHost[0] != "www") {
		        if (arHost[arHost.length-1].length > 2 && arHost[arHost.length-2].length > 3) {
		            wwwHost = arCurrentAddy[0] + "//www" + "."  + arHost[1] + "." + arHost[2] + "/";
					this.updateUpAppendItem(upContainer, wwwHost);
				}
	        }
	        // Assumes something like this www.cs.insertschoolhere.edu, up converts to www.insertschoolhere.edu
	        // ignores www.somewhere.co.uk or www.somewhere.com.au or similar
	        else if (arHost.length == 4 && arHost[0] == "www") {
		        if (arHost[arHost.length-1].length > 2 && arHost[arHost.length-2].length > 3) {
		            wwwHost = arCurrentAddy[0] + "//www" + "."  + arHost[arHost.length-2] + "." + arHost[arHost.length-1] + "/";
					this.updateUpAppendItem(upContainer, wwwHost);
				}
	        }
	        // Assumes something like this sub.subdomain.somewhere.com
	        // or www.subdomain.somewhere.co.uk or subdomain.somewhere.co.uk
	        // or subdomain.somewhere.com.au or subdomain.nowhere.org.jp or similar
	        else if (arHost.length >= 4) {
	        	// sub.subdomain.somewhere.com -> www.somewhere.com
	        	// www.sub.subdomain.somewhere.com -> www.somewhere.com
	        	// you get the picture...
	            if (arHost[arHost.length-1].length > 2 && arHost[arHost.length-2].length > 3) {
				    wwwHost = arCurrentAddy[0] + "//www" + "."  + arHost[arHost.length-2] + "." + arHost[arHost.length-1] + "/";
					this.updateUpAppendItem(upContainer, wwwHost);
	            }
	            // www.subdomain.somewhere.co.uk -> www.somewhere.co.uk
	            // sub.subdomain.somewhere.co.uk -> www.somewhere.co.uk
	            // www.sub.subdomain.nowhere.org.jp -> www.nowhere.org.jp
	            // you get the picture...
	            else {
				    wwwHost = arCurrentAddy[0] + "//www" + "."  + arHost[arHost.length-3] + "."  + arHost[arHost.length-2] + "." + arHost[arHost.length-1] + "/";
					this.updateUpAppendItem(upContainer, wwwHost);
	            }
	        }
	    }
	
	    if (upContainer.childNodes.length > 0) {
	        myGooglebarUtil.getElement("ggb_UpDirButton").setAttribute("disabled", false);
	    }
	    else {
	        myGooglebarUtil.getElement("ggb_UpDirButton").setAttribute("disabled", true);
	    }
	},
	
	updateUpAppendItem: function(aContainer, aValue) {
        var tempNode = document.createElement("menuitem");
        tempNode.setAttribute("label", aValue);
        tempNode.setAttribute("class", "ggb_css_menuitem_all");
        tempNode.setAttribute("oncommand", "myGooglebar.upGo(event, '" + aValue + "'); event.preventBubble();");
        tempNode.setAttribute("tooltiptext", aValue);
        aContainer.appendChild(tempNode);	
	},
	
	updateUniv: function() {
	    window._content.document.location = GB_UNIV_URL;
	},
	
	upGo: function(aEvent, aAddy) {
	    this.doAction(aAddy, this.getLocFromEvent(aEvent));
	},
	
	upGoOneLevel: function(aEvent) {
	    var mode;
	    var win = window._content.document;
	
	    var upContainer = myGooglebarUtil.getElement("ggb_up_dir_popup");
	
	    if (upContainer && upContainer.childNodes.length > 0) {
	        this.doAction(upContainer.childNodes.item(0).getAttribute("label"), this.getLocFromEvent(aEvent));
	    }
	}
};
